



ckeygen --help
IF %ERRORLEVEL% NEQ 0 exit 1
pyhtmlizer --help
IF %ERRORLEVEL% NEQ 0 exit 1
tkconch --help
IF %ERRORLEVEL% NEQ 0 exit 1
trial --help
IF %ERRORLEVEL% NEQ 0 exit 1
twist --help
IF %ERRORLEVEL% NEQ 0 exit 1
twistd --help
IF %ERRORLEVEL% NEQ 0 exit 1
exit 0
